@@ -12,29 +12,30 @@ import warnings
 import functools
 import itertools
 import posixpath
-import contextlib
 import collections
 
-from ._collections import FreezableDefaultDict
+from . import _adapters, _meta
+from ._collections import FreezableDefaultDict, Pair
 from ._compat import (
     NullFinder,
-    Protocol,
     PyPy_repr,
     install,
 )
 from ._functools import method_cache
 from ._itertools import unique_everseen
+from ._meta import PackageMetadata, SimplePath
 
 from contextlib import suppress
 from importlib import import_module
 from importlib.abc import MetaPathFinder
 from itertools import starmap
-from typing import Any, List, Mapping, Optional, TypeVar, Union
+from typing import List, Mapping, Optional, Union
 
 
 __all__ = [
     'Distribution',
     'DistributionFinder',
+    'PackageMetadata',
     'PackageNotFoundError',
     'distribution',
     'distributions',
@@ -64,17 +65,27 @@ class Sectioned:
     """
     A simple entry point config parser for performance
 
-    >>> res = Sectioned.get_sections(Sectioned._sample)
-    >>> sec, values = next(res)
-    >>> sec
+    >>> for item in Sectioned.read(Sectioned._sample):
+    ...     print(item)
+    Pair(name='sec1', value='# comments ignored')
+    Pair(name='sec1', value='a = 1')
+    Pair(name='sec1', value='b = 2')
+    Pair(name='sec2', value='a = 2')
+
+    >>> res = Sectioned.section_pairs(Sectioned._sample)
+    >>> item = next(res)
+    >>> item.name
     'sec1'
-    >>> [(key, value) for key, value in values]
-    [('a', '1'), ('b', '2')]
-    >>> sec, values = next(res)
-    >>> sec
+    >>> item.value
+    Pair(name='a', value='1')
+    >>> item = next(res)
+    >>> item.value
+    Pair(name='b', value='2')
+    >>> item = next(res)
+    >>> item.name
     'sec2'
-    >>> [(key, value) for key, value in values]
-    [('a', '2')]
+    >>> item.value
+    Pair(name='a', value='2')
     >>> list(res)
     []
     """
@@ -91,33 +102,29 @@ class Sectioned:
         """
     ).lstrip()
 
-    def __init__(self):
-        self.section = None
-
-    def __call__(self, line):
-        if line.startswith('[') and line.endswith(']'):
-            # new section
-            self.section = line.strip('[]')
-            return
-        return self.section
-
     @classmethod
-    def get_sections(cls, text):
-        lines = filter(cls.valid, map(str.strip, text.splitlines()))
+    def section_pairs(cls, text):
         return (
-            (section, map(cls.parse_value, values))
-            for section, values in itertools.groupby(lines, cls())
-            if section is not None
+            section._replace(value=Pair.parse(section.value))
+            for section in cls.read(text, filter_=cls.valid)
+            if section.name is not None
         )
 
+    @staticmethod
+    def read(text, filter_=None):
+        lines = filter(filter_, map(str.strip, text.splitlines()))
+        name = None
+        for value in lines:
+            section_match = value.startswith('[') and value.endswith(']')
+            if section_match:
+                name = value.strip('[]')
+                continue
+            yield Pair(name, value)
+
     @staticmethod
     def valid(line):
         return line and not line.startswith('#')
 
-    @staticmethod
-    def parse_value(line):
-        return map(str.strip, line.split("=", 1))
-
 
 class EntryPoint(
     PyPy_repr, collections.namedtuple('EntryPointBase', 'name value group')
@@ -255,9 +262,8 @@ class EntryPoints(tuple):
     @staticmethod
     def _parse_groups(text):
         return (
-            (name, value, section)
-            for section, values in Sectioned.get_sections(text)
-            for name, value in values
+            (item.value.name, item.value.value, item.name)
+            for item in Sectioned.section_pairs(text)
         )
 
 
@@ -388,25 +394,6 @@ class FileHash:
         return '<FileHash mode: {} value: {}>'.format(self.mode, self.value)
 
 
-_T = TypeVar("_T")
-
-
-class PackageMetadata(Protocol):
-    def __len__(self) -> int:
-        ...  # pragma: no cover
-
-    def __contains__(self, item: str) -> bool:
-        ...  # pragma: no cover
-
-    def __getitem__(self, key: str) -> str:
-        ...  # pragma: no cover
-
-    def get_all(self, name: str, failobj: _T = ...) -> Union[List[Any], _T]:
-        """
-        Return all values associated with a possibly multi-valued key.
-        """
-
-
 class Distribution:
     """A Python distribution package."""
 
@@ -491,7 +478,7 @@ class Distribution:
         return PathDistribution(zipp.Path(meta.build_as_zip(builder)))
 
     @property
-    def metadata(self) -> PackageMetadata:
+    def metadata(self) -> _meta.PackageMetadata:
         """Return the parsed metadata for this Distribution.
 
         The returned object will have keys that name the various bits of
@@ -505,7 +492,7 @@ class Distribution:
             # (which points to the egg-info file) attribute unchanged.
             or self.read_text('')
         )
-        return email.message_from_string(text)
+        return _adapters.Message(email.message_from_string(text))
 
     @property
     def name(self):
@@ -573,24 +560,7 @@ class Distribution:
 
     @classmethod
     def _deps_from_requires_text(cls, source):
-        section_pairs = cls._read_sections(source.splitlines())
-        sections = {
-            section: list(map(operator.itemgetter('line'), results))
-            for section, results in itertools.groupby(
-                section_pairs, operator.itemgetter('section')
-            )
-        }
-        return cls._convert_egg_info_reqs_to_simple_reqs(sections)
-
-    @staticmethod
-    def _read_sections(lines):
-        section = None
-        for line in filter(None, lines):
-            section_match = re.match(r'\[(.*)\]$', line)
-            if section_match:
-                section = section_match.group(1)
-                continue
-            yield locals()
+        return cls._convert_egg_info_reqs_to_simple_reqs(Sectioned.read(source))
 
     @staticmethod
     def _convert_egg_info_reqs_to_simple_reqs(sections):
@@ -615,9 +585,8 @@ class Distribution:
             conditions = list(filter(None, [markers, make_condition(extra)]))
             return '; ' + ' and '.join(conditions) if conditions else ''
 
-        for section, deps in sections.items():
-            for dep in deps:
-                yield dep + parse_condition(section)
+        for section in sections:
+            yield section.value + parse_condition(section.name)
 
 
 class DistributionFinder(MetaPathFinder):
@@ -649,10 +618,11 @@ class DistributionFinder(MetaPathFinder):
         @property
         def path(self):
             """
-            The path that a distribution finder should search.
+            The sequence of directory path that a distribution finder
+            should search.
 
-            Typically refers to Python package paths and defaults
-            to ``sys.path``.
+            Typically refers to Python installed package paths such as
+            "site-packages" directories and defaults to ``sys.path``.
             """
             return vars(self).get('path', sys.path)
 
@@ -702,7 +672,7 @@ class FastPath:
 
     @property
     def mtime(self):
-        with contextlib.suppress(OSError):
+        with suppress(OSError):
             return os.stat(self.root).st_mtime
         self.lookup.cache_clear()
 
@@ -814,11 +784,10 @@ class MetadataPathFinder(NullFinder, DistributionFinder):
 
 
 class PathDistribution(Distribution):
-    def __init__(self, path):
-        """Construct a distribution from a path to the metadata directory.
+    def __init__(self, path: SimplePath):
+        """Construct a distribution.
 
-        :param path: A pathlib.Path or similar object supporting
-                     .joinpath(), __div__, .parent, and .read_text().
+        :param path: SimplePath indicating the metadata directory.
         """
         self._path = path
 
@@ -855,7 +824,7 @@ def distributions(**kwargs):
     return Distribution.discover(**kwargs)
 
 
-def metadata(distribution_name) -> PackageMetadata:
+def metadata(distribution_name) -> _meta.PackageMetadata:
     """Get the metadata for the named package.
 
     :param distribution_name: The name of the distribution package to query.
@@ -912,7 +881,7 @@ def requires(distribution_name):
     Return a list of requirements for the named package.
 
     :return: An iterator of requirements, suitable for
-    packaging.requirement.Requirement.
+        packaging.requirement.Requirement.
     """
     return distribution(distribution_name).requires
 
